package com.sd.business.service.bom.impl;

import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.bom.dto.BomSpecPriceQueryDto;
import com.sd.business.entity.bom.enums.BomSpecPriceConfigCodeEnum;
import com.sd.business.entity.bom.po.BomSpecPriceConfig;
import com.sd.business.mapper.bom.BomSpecPriceConfigMapper;
import com.sd.business.service.bom.BomSpecPriceConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import com.sd.business.entity.bom.vo.BomSpecPriceConfigVo;
import com.sd.business.entity.bom.dto.BomSpecPriceConfigSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.dto.BomSpecPriceConfigDto;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * bom规格价格配置 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-12-25
 */
@Service
public class BomSpecPriceConfigServiceImpl extends ServiceImpl<BomSpecPriceConfigMapper, BomSpecPriceConfig> implements BomSpecPriceConfigService {

    @Override
    public List<BomSpecPriceConfigVo> getList(BomSpecPriceConfigSelectDto dto) {
        IWrapper<BomSpecPriceConfig> wrapper = getWrapper();
        wrapper.orderByDesc("bspc", BomSpecPriceConfig::getId);
        wrapper.eq("bspc", BomSpecPriceConfig::getCode, dto.getCode());
        List<BomSpecPriceConfigVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public void add(BomSpecPriceConfigDto bomSpecPriceConfigDto) {
        this.save(bomSpecPriceConfigDto);
    }

    @Override
    public void edit(BomSpecPriceConfigDto bomSpecPriceConfigDto) {
        this.updateById(bomSpecPriceConfigDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public BigDecimal getBomSpecPrice(BomSpecPriceQueryDto dto) {
        // 计算公式=（基材价格+品类+特性+颜色价格）*[（长+裁切配置）*（宽+裁切配置）/10000]*（厚度+压纹配置）*（1+损耗）+加工费+运费
        List<BomSpecPriceConfig> list = this.list();
        Map<Long, BomSpecPriceConfig> configDetailMap = list.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        Map<String, List<BomSpecPriceConfig>> configCodeMap = list.stream().collect(Collectors.groupingBy(BomSpecPriceConfig::getCode));

        // 计算bom每平方价格
        BigDecimal price = BigDecimal.ZERO;
        // 等级
        BomSpecPriceConfig levelDetail = configDetailMap.get(dto.getLevelConfigId());
        price = price.add(levelDetail.getPrice());

        // 品类
        BomSpecPriceConfig categoryDetail = configDetailMap.get(dto.getCategoryConfigId());
        price = price.add(categoryDetail.getPrice());

        // 特性
        if (StrUtil.isNotBlank(dto.getCharacterConfigIds())) {
            String[] characterConfigIdArr = dto.getCharacterConfigIds().split(",");
            for (String configId : characterConfigIdArr) {
                BomSpecPriceConfig embossingDetail = configDetailMap.get(Long.parseLong(configId));
                price = price.add(embossingDetail.getPrice());
            }
        }

        // 颜色
        String colourConfigIds = dto.getColourConfigIds();
        List<String> colourConfigIdList = Arrays.asList(colourConfigIds.split(","));
        for (String configId : colourConfigIdList) {
            BomSpecPriceConfig colourDetail = configDetailMap.get(Long.parseLong(configId));
            BigDecimal colourPrice = colourDetail.getPrice();
            if (colourConfigIdList.size() > 1) {
                colourPrice = colourPrice.divide(BigDecimal.valueOf(2), 2, RoundingMode.HALF_UP);
            }
            price = price.add(colourPrice);
        }

        // 获取裁切配置
        List<BomSpecPriceConfig> bomSpecCropConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.CROP_CONFIG.getKey(), Collections.emptyList());
        BomSpecPriceConfig cropConfig = bomSpecCropConfigList.stream()
                .filter(item -> item.getName().contains(categoryDetail.getName()))
                .findAny().orElse(null);
        if (cropConfig != null) {
            dto.setLength(dto.getLength().add(cropConfig.getLength()));
            dto.setWidth(dto.getWidth().add(cropConfig.getWidth()));
        }

        // 获取压纹配置
        List<BomSpecPriceConfig> bomSpecEmbossingConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.EMBOSSING_CONFIG.getKey(), Collections.emptyList());
        BomSpecPriceConfig embossingConfig = bomSpecEmbossingConfigList.stream()
                .filter(item -> item.getName().contains(dto.getEmbossing()))
                .findAny().orElse(null);
        if (embossingConfig != null) {
            dto.setHeight(dto.getHeight().add(embossingConfig.getHeight()));
        }

        // 获取损耗配置
        BigDecimal depletion = BigDecimal.ZERO;
        String depletionCode;
        if (Objects.equals(dto.getBomClassify(), StatusConstant.YES)) {
            depletionCode = BomSpecPriceConfigCodeEnum.YOGA_MAT_DEPLETION_CONFIG.getKey();
        } else {
            depletionCode = BomSpecPriceConfigCodeEnum.SKIP_MAT_DEPLETION_CONFIG.getKey();
        }
        List<BomSpecPriceConfig> bomSpecDepletionConfigList = configCodeMap.getOrDefault(depletionCode, Collections.emptyList());
        BomSpecPriceConfig depletionConfig = bomSpecDepletionConfigList.stream()
                .filter(item -> item.getName().contains(categoryDetail.getName()))
                .findAny().orElse(null);
        if (depletionConfig != null) {
            depletion = depletionConfig.getDepletion();
        }
        // 获取附加配置
        List<BomSpecPriceConfig> additionalConfigList = bomSpecDepletionConfigList.stream()
                .filter(item -> Objects.equals(item.getType(), StatusConstant.YES)).collect(Collectors.toList());

        // 瑜伽垫对比宽度，跳绳垫对比级别
        BomSpecPriceConfig additionalConfig;
        if (Objects.equals(dto.getBomClassify(), StatusConstant.YES)) {
            additionalConfig = additionalConfigList.stream()
                    .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth()))
                    .findAny().orElse(null);
        } else {
            additionalConfig = additionalConfigList.stream()
                    .filter(item -> item.getName().contains(levelDetail.getName()))
                    .findAny().orElse(null);
        }

        // 附加配置匹配成功，增加损耗
        if (additionalConfig != null) {
            depletion = depletion.add(additionalConfig.getDepletion());
        }

        // 获取加工费配置
        List<BomSpecPriceConfig> bomSpecProcessingFeeConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.PROCESSING_FEE_CONFIG.getKey(), Collections.emptyList());
        BigDecimal processingFee = bomSpecProcessingFeeConfigList.stream()
                .map(BomSpecPriceConfig::getPrice)
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 获取运费配置
        List<BomSpecPriceConfig> bomSpecFreightConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.FREIGHT_CONFIG.getKey(), Collections.emptyList());
        BigDecimal freightFee = BigDecimal.ZERO;
        BomSpecPriceConfig freightConfig = bomSpecFreightConfigList.stream()
                .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth())).findAny().orElse(null);
        if (freightConfig != null) {
            freightFee = freightConfig.getPrice();
        }

        return price.multiply(dto.getLength().multiply(dto.getWidth()).divide(BigDecimal.valueOf(10000), 2, RoundingMode.HALF_UP))
                .multiply(dto.getHeight())
                .multiply(BigDecimal.ONE.add(depletion.multiply(BigDecimal.valueOf(0.01))))
                .add(processingFee)
                .add(freightFee)
                .setScale(2, RoundingMode.HALF_UP);
    }

    /**
     * 对比
     * @param rule 规则
     * @param value
     * @return
     */
    private boolean operatorCompareTo(String rule, BigDecimal value) {
        Integer number = ReUtil.getFirstNumber(rule);
        if (rule.startsWith(">=") || rule.startsWith("≥")) {
            return value.compareTo(BigDecimal.valueOf(number)) >= 0;
        } else if (rule.startsWith(">")) {
            return value.compareTo(BigDecimal.valueOf(number)) > 0;
        } else if (rule.startsWith("<=") || rule.startsWith("≤")) {
            return value.compareTo(BigDecimal.valueOf(number)) <= 0;
        } else if (rule.startsWith("<")) {
            return value.compareTo(BigDecimal.valueOf(number)) < 0;
        } else {
            return value.compareTo(BigDecimal.valueOf(number)) == 0;
        }
    }

}
